import pytest
from selenium import webdriver
from tlib.base import AndroidHelper
from tlib.base import TestHelper
from tlib.base.TestHelper import webdriver_chrome_executable, webdriver_ie_executable

def pytest_addoption(parser):
    """
    Options supported by pytest's selenium plug-in
    """
    group = parser.getgroup("selenium", "Testing with the selenium framework")
    group.addoption(
        "--base_url",
        action="store",
        dest="base_url",
        help="base url of the server used for testing")

    group.addoption(
        "--mobile_base_url",
        action="store",
        dest="mobile_base_url",
        help="mobile base url of the server used for testing")

    group.addoption(
        "--browser",
        action="append",
        dest="browser",
        choices=["chrome", "firefox", "ie", "android"],
        help="Browser used for running test cases.")

    group.addoption(
        "--proxy",
        action="store",
        dest="proxy",
        help="""Use provided IP and port as proxy server
            Sample usage:
            py.test --proxy=127.0.0.1:8080""")

    group.addoption(
        "--serial_id",
        action="store",
        dest="serial_id",
        help="Serial of the android device or simulator where Android tests will be run")


def pytest_generate_tests(metafunc):
    """
    Parses command line --browser and parametrizes tlib_tests with each browser specified
    """
    browsers = metafunc.config.getoption("browser")
    if browsers is not None:
        metafunc.parametrize("browser", browsers, indirect=True, scope="class")

#noinspection PyShadowingNames,PyUnresolvedReferences
@pytest.fixture(scope='session')
def browser(request, tlib_logger, adb_logger):
    """
    Creates a browser as specified by the command line parameter --browser and --proxy\n
    Note: Currently proxy parameter is ignored
    """
    global b

    #Don't create a browser object is --browser was not specified
    if not hasattr(request, 'param') or request.param is None:
        tlib_logger.warn("Parameter browser was not specified, skipping test")
        pytest.skip("Parameter browser was not specified, skipping test")
    else:
        if request.param == "ie":
            tlib_logger.debug("Creating webdriver for IE")
            b = webdriver.Ie(webdriver_ie_executable())
            request.addfinalizer(b.quit)
        elif request.param == "chrome":
            tlib_logger.debug("Creating webdriver for Chrome")
            b = webdriver.Chrome(webdriver_chrome_executable())
            request.addfinalizer(b.quit)
        elif request.param == "firefox":
            tlib_logger.debug("Creating webdriver for Firefox")
            b = webdriver.Firefox()
            request.addfinalizer(b.quit)
        elif request.param == "android":
            tlib_logger.debug("Creating webdriver for Android")

            serial_id = request.config.getoption("serial_id")
            if serial_id is None:
                tlib_logger.warn("Parameter --serial_id was not given. Running test on default device")

            #Start android server
            AndroidHelper.setup_webdriver(tlib_logger, adb_logger, serial_id)
            b = webdriver.Remote(command_executor='http://localhost:8080/wd/hub',  desired_capabilities=webdriver.DesiredCapabilities.ANDROID)

            def fin():
                b.quit
                AndroidHelper.teardown_webdriver(tlib_logger, adb_logger, serial_id)

            request.addfinalizer(fin)

        else:
            #noinspection PyUnresolvedReferences
            tlib_logger.warn("{browser}s is not a valid browser".format(browser=request.param))
            pytest.skip("{browser}s is not a valid browser".format(browser=request.param))


    return b

#noinspection PyUnresolvedReferences
@pytest.fixture(scope="session")
def base_url(request, tlib_logger):
    """
    Returns value of --base_url command line parameter
    """
    if request.config.getoption("base_url") is None:
        #noinspection PyUnresolvedReferences
        tlib_logger.warn("parameter --base_url was not specified")
        pytest.skip("parameter --base_url was not specified")
        return None
    else:
        return request.config.getoption("base_url")

#noinspection PyUnresolvedReferences
@pytest.fixture(scope="session")
def mobile_base_url(request, tlib_logger):
    """
    Returns value of --mobile_base_url command line parameter
    """
    if request.config.getoption("mobile_base_url") is None:
        #noinspection PyUnresolvedReferences
        tlib_logger.warn("parameter --mobile_base_url was not specified")
        pytest.fail("parameter --mobile_base_url was not specified")
        return None
    else:
        return request.config.getoption("mobile_base_url")

#noinspection PyUnresolvedReferences
@pytest.fixture(scope="session")
def proxy(request, tlib_logger):
    """
    Returns value of --proxy command line parameter
    """
    if request.config.getoption("proxy") is None:
        tlib_logger.info("Parameter --proxy was not specified")
        return None
    else:
        p = request.config.getoption("proxy")
        global port
        global ip
        #noinspection PyBroadException
        try:
            (ip, port) = p.split(":")
        except:
            tlib_logger.fatal("Invalid value for --proxy parameter: {proxy}. Should be an IP and port like this 172.27.108.99:8080")
            #noinspection PyUnresolvedReferences
            pytest.exit("""Invalid value for --proxy parameter: {proxy}
                Should be an IP and port like this 172.27.108.99:8080""")

        #validate ip is valid
        if not TestHelper.is_valid_ip(ip):
            tlib_logger.fatal("Proxy server has an invalid IP address: {ip}".format(ip=ip))
            #noinspection PyUnresolvedReferences
            pytest.exit("Proxy server has an invalid IP address: {ip}".format(ip=ip))

        #Validate port is an integer between 1 and 65535
        #noinspection PyBroadException
        try:
            port = int(port, base=10)
        except:
            tlib_logger.fatal("Proxy server has an invalid port: (port)".format(port=port))
            #noinspection PyUnresolvedReferences
            pytest.exit("Proxy server has an invalid port: (port)".format(port=port))

        if int(port) not in range(1, 65535, 1):
            tlib_logger.fatal("Proxy server has an invalid port: (port)".format(port=port))
            #noinspection PyUnresolvedReferences
            pytest.exit("Proxy server has an invalid port: (port)".format(port=port))

        return p