import os

__author__ = 'AOsorio1'

# noinspection PyPackageRequirements
import pytest
# noinspection PyPackageRequirements
from selenium import webdriver
from _pytest.python import FixtureRequest
from _pytest.mark import MarkInfo
# noinspection PyPackageRequirements
import jinja2
from tlib.base import TestHelper
from tlib.base.PytestTester import PytestTester
from selenium.webdriver.support.ui import WebDriverWait
from selenium.common.exceptions import NoSuchElementException, ElementNotVisibleException, TimeoutException
import logging


# noinspection PyMethodParameters
# noinspection PyUnresolvedReferences
class SeleniumTester(PytestTester):

    homepage = ""

    _test_logger = None # Reference to test logger
    _browser = None

    _test_case_id = None
    _test_params = None

    _jinja_env = jinja2.Environment(loader = jinja2.FileSystemLoader(TestHelper.tlib_template_folder()), trim_blocks = True)

    _screenshot_report = {} # List of tests for which there are screenshots
    _screenshot_report_template = _jinja_env.get_template("screenshot_report.html") # Template for generating screenshot report

    _screenshots = {} # Screenshots generated during a test case
    _tc_screenshot_template = _jinja_env.get_template("testcase_screenshots.html") # Template for generating screenshots for a test case


    @property
    def browser(self):
        """
        Instance of the WebDriver browser
        @type: webdriver.Remote
        """
        return self.__class__._browser


    @property
    def test_logger(self):
        """
        Logger object to log test information
        @type: logging
        """
        return self.__class__._test_logger


    @staticmethod
    def screenshot_folder():
        """
        Returns project's screenshot folder
        This method must be defined in subclasses
        """
        raise NotImplementedError("Subclasses should implement this!")


    @pytest.fixture(scope='class', autouse=True)
    def initialize_class(self, test_logger, browser):
        """
        @type test_logger: logging
        @type browser: webdriver.Remote
        Stores an instance of browser and test_logger
        """
        self.__class__._test_logger = test_logger
        self.__class__._browser = browser


    @pytest.fixture(scope='class', autouse=True)
    def generate_screenshot_report(self, request):
        """
        Generates HTML page with all test case results for the class
        @type request: FixtureRequest
        """
        self.__class__._screenshot_report = {}

        def generate_report():
            if len(self.__class__._screenshot_report) > 0:
                #Generate HTML based on template
                html = self.__class__._screenshot_report_template.render(test_class=self.__class__.__name__,
                            test_case_id=self.__class__._test_case_id, files=self.__class__._screenshot_report)

                html_file = "%s.html" % self.__class__.__name__
                full_filename = os.path.join(self.screenshot_folder(), html_file)

                f = open(full_filename, "w")
                try:
                    f.write(html)
                finally:
                    f.close()

        request.addfinalizer(generate_report)


    @pytest.fixture(scope='function', autouse=True)
    def generate_test_case_screenshot(self, request):
        """
        Generates HTML page with screenshots for each test
        @type request: FixtureRequest
        """
        self.__class__._test_params = request.keywords.node._genid
        self.__class__._screenshots = {}

        #Get marker testcasename or use a default if there is none
        marker = request.node.get_marker("testcasename")
        if marker is None:
            self.__class__._test_case_id = "UNKNOWN_TEST_CASE_ID"
        else:
            self.__class__._test_case_id = marker.args[0]


        def generate_screenshots():
            if len(self.__class__._screenshots) > 0:
                #Generate HTML based on template
                html = self.__class__._tc_screenshot_template.render(test_case_id=self.__class__._test_case_id,
                            test_case_name=request.keywords.node.name,
                            screenshots=self.__class__._screenshots)

                html_file = "%s.html" % self.__class__._test_params
                relative_filename = os.path.join(self.__class__._test_case_id, html_file)
                full_filename = os.path.join(self.screenshot_folder(), relative_filename)

                f = open(full_filename, "w")
                try:
                    f.write(html)
                finally:
                    f.close()

                #Append current screenshot file to list of screenshots
                self.__class__._screenshot_report[request.keywords.node.name] = relative_filename

        request.addfinalizer(generate_screenshots)


    def navigate(self, page, save_screenshot=True):
        """
        Navigate to "page"
        """
        self.test_logger.info("Navigating to %s" % page)
        self.browser.get(page)
        if save_screenshot:
            self.save_screenshot("Navigate to %s" % page)


    def go_home(self):
        """
        Navigate to home page
        """
        self.navigate(self.homepage, save_screenshot=False)
        self.save_screenshot("Homepage")


    def save_screenshot(self, description):
        """
        Saves screen shot for the current page
        """
        #Waits until page is loaded
        self.wait_for_page_loaded()

        #Folder containing images for test case
        test_folder = os.path.join(self.screenshot_folder(), self.__class__._test_case_id, self.__class__._test_params)

        try:
            os.makedirs(test_folder)
        except WindowsError:
            pass

        #Get number of existing images
        i = len(os.listdir(test_folder)) + 1

        #Get filename
        img_filename = "%02d.jpg" % i
        full_path = os.path.join(test_folder, img_filename)

        self.__class__._screenshots[i] = {"filename": "%s/%s" % (self.__class__._test_params, img_filename),
                                          "description": description}

        self.browser.get_screenshot_as_file(full_path)


    def wait_for_page_loaded(self, timeout=10):
        """
        Waist until document.readyState is equal to complete
        """
        condition = lambda *args: self.browser.execute_script("return document.readyState") == "complete"
        try:
            WebDriverWait(self.browser, timeout*1000).until(condition)
        except TimeoutException:
            self.test_logger.error('Timeout while waiting for page to load')
            pytest.fail('Timeout while waiting for page to load')


    @pytest.fixture(scope='function', autouse=True)
    def setup_go_home(self, request):
        """
        Goes to homepage before each test, unless marker skipsetup is given
        """
        #Get all markers
        marks = {}
        for k,v in request.node.function.__dict__.iteritems():
            if isinstance(v, MarkInfo):
                marks[k] = v

        #Don't go to home is skipsetup marker was given
        if not marks.has_key('skipsetup'):
            self.go_home()
        else:
            self.test_logger.info("Skipping setup")

